// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"

namespace Dawn {
  template<typename T, typename L>
  class IShaderParameterBuffer {
    public:
      /**
       * Initializes this shader parameter buffer.
       */
      virtual void init() = 0;

      /**
       * Basic buffer method. Buffers the entire contents of the data struct to
       * this shader parameter buffer.
       * 
       * @param data Data to buffer to the parameter.
       */
      virtual void buffer(T *data) = 0;

      /**
       * Bind this shader buffer to the supplied location.
       * 
       * @param location Location to bind this buffer to.
       */
      virtual void bind(L location) = 0;
  };
}